class ApiResponseException(Exception):
    """Exception raised when api response is not 200 Ok

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class MissingArgumentException(Exception):
    """Exception raised when obligatory argument is not provided

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



import io
import os
import time

import requests
from urllib3.util.retry import Retry
from .utils import log
from .Authenticator import Authenticator
from .constants import USER_AGENT, API_ENDPOINT
from .exceptions import ApiResponseException
from .utils import is_blank


class HTTPCaller:
    def __init__(self):
        self.jobid = os.environ.get('JOBID')
        self.eid = os.environ.get('EID')
        self.api_url = os.environ.get('NG_API_ENDPOINT', API_ENDPOINT)
        self.url = self.api_url
        self.user_agent = USER_AGENT

    def _set_headers(self, headers=None):
        if headers is None:
            headers = {}
        # adding token to the headers
        token = Authenticator().get_token()
        headers.update(token)
        if not is_blank(self.eid):
            headers.update({'X-KH-E-ID': self.eid})
        if not is_blank(self.jobid):
            headers.update({'X-KH-JOB-ID': self.jobid})
        if not is_blank(self.user_agent):
            headers.update({'User-Agent': self.user_agent})
        return headers

    def post(self, path, payload=None, data=None, headers=None):
        headers = self._set_headers(headers)

        log.debug(f'Sending POST request to {path}, with payload: {payload}')
        r = requests.post(self.url + path, json=payload, data=data, headers=headers)
        if (r.status_code >= 200) and (r.status_code < 300):
            log.info('Posting with message {} was successful with response: {}'.format(payload, r.text))
            return r

        else:
            exception_msg = 'Posting with message {} ended with error (error code: {}). Response: {}' \
                .format(payload,
                        r.status_code,
                        r.text)
            log.error(
                exception_msg)
            raise ApiResponseException(exception_msg)

    def get(self, path, headers=None, query_params: dict = None):
        headers = self._set_headers(headers)

        # Request URL
        full_path = self.url + path
        log.debug('GET request before sending: {}, {}'.format(full_path, headers))

        s = self._create_session()

        r = s.get(full_path, headers=headers, params=query_params)

        log.debug(f'GET: {r.url}')
        log.debug(f'Headers: {r.request.headers}')

        log.debug('>> GET method returned a {} status'.format(r.status_code))

        if (r.status_code >= 200) and (r.status_code < 300):
            log.debug('Data received from {} with response: {}'.format(path, r.text))
            return r

        else:
            exception_msg = 'Data reception from {} ended with error (error code: {}). Response: {}' \
                .format(path, r.status_code, r.text)
            log.error(
                exception_msg)
            raise ApiResponseException(exception_msg)

    def file_uploader(self, payload, file, headers=None, verify_upload=True):
        '''
        Uploads a file from disk or an object from RAM to datalake with a PUT function

        :param payload:
        :param file: python object to stream or path to a disk file to upload
        :param headers:
        :param verify_upload: if false will not verify upload success
        :return:
        '''

        headers = self._set_headers(headers)

        # send meta data
        file_params_json = self.post('/file', payload, headers=headers).json()
        log.debug(file_params_json)
        url = file_params_json['location']
        file_id = file_params_json['fileId']

        # Get the file content to upload as binary stream
        if isinstance(file, str):
            file_content = open(file, 'rb').read()

        elif isinstance(file, io.BytesIO):
            # Stream the python object directly
            file_content = file

        else:
            raise TypeError(
                'Passed file argument must be either a file path to a file saved on the disk, '
                'either a io.BytesIO object to stream from memory')

        # PUT request
        r = requests.put(url,
                         data=file_content,
                         headers={'Content-Type': 'application/octet-stream', "User-Agent": self.user_agent}
                         )

        log.debug(f'PUT {r.url} \nHEADERS: {r.request.headers}')

        avoid_duplicates = self.is_avoid_duplicates(payload)
        # Returned code after uploading
        if (r.status_code >= 200) and (r.status_code < 300):
            # wait and ping every 10 seconds where the upload is finished - up to 10 times
            job_id = headers.get("X-KH-JOB-ID")
            if verify_upload:
                if not self.check_upload_successful(avoid_duplicates, job_id, file_id, 10):
                    raise Exception(f"File {file_id} upload is not successful after 100 seconds of waiting")

        return file_id

    def check_upload_successful(self, avoid_duplicates: bool, job_id: str, file_id: str,
                                uploaded_file_check_retries: int):
        i = 0
        while i < uploaded_file_check_retries:
            status_response = self.get('/status?jobId={}'.format(job_id))
            status_response_body = status_response.json()
            i += 1
            log.debug(f'Search the uploaded file for the {i} time')
            log.debug(status_response_body['items'])
            items = status_response_body['items']

            for item in items:
                val = self.__check_item(item, avoid_duplicates, file_id)
                if val is None:
                    if i == uploaded_file_check_retries:
                        return False
                    time.sleep(10)
                    continue

                elif val:
                    return True

                else:
                    return False
        return False

    def __check_item(self, item, avoid_duplicates: bool, file_id: str):
        if item is None:
            return False

        for cs in item.get('componentStatuses', []):
            payload = cs.get('payload', {})
            status = cs.get('status', None)
            s_file_id = payload.get('fileId', None)
            if file_id == s_file_id:
                # duplicate
                if status == 'WARN':
                    if avoid_duplicates:
                        log.info('File Duplicated')
                        return True
                    else:
                        return False

                # file uploaded
                if status == 'INFO':
                    log.info('File uploaded')
                    return True

        return None

    @staticmethod
    def _create_session():
        s = requests.Session()
        # Define your retries for http and https urls

        retries = Retry(backoff_factor=0.1, total=5, status_forcelist=[104, 413, 429, 503, 502, 500])
        # Create adapters with the retry logic for each

        retry_adapter = requests.adapters.HTTPAdapter(max_retries=retries)
        # Replace the session�s original adapters
        s.mount('http://', retry_adapter)
        s.mount('https://', retry_adapter)
        return s

    @staticmethod
    def is_avoid_duplicates(payload):
        is_avoid_duplicates = False
        if 'fields' not in payload:
            return False

        for field in payload['fields']:
            if field['name'] is not None and field['name'] == 'AVOID_DUPLICATES':
                is_avoid_duplicates = eval((field['value']).capitalize())
        return is_avoid_duplicates

    @staticmethod
    def get_from_s3(s3_path):

        s = HTTPCaller._create_session()
        r = s.get(s3_path)

        log.debug(f'GET form S3: {s3_path}')

        if (r.status_code >= 200) and (r.status_code < 300):
            log.debug('File downloaded from {}'.format(s3_path))
            return r

        else:
            log.error(
                'File download from {} ended with error (error code: {}). Response: {}'.format(s3_path, r.status_code,
                                                                                               r.text))
            return r



import os
import unittest
from generalindex import Timeseries

os.environ["NG_API_AUTHTOKEN"] = "test"


class TimeseriesTestCase(unittest.TestCase):
    def test_given_wrong_timezone_when_verify_then_raise_exc(self):
        with self.assertRaises(ValueError) as context:
            Timeseries.verify_timezone("UFX")

        self.assertTrue(
            'Wrongly formatted timezone argument. Must be for example Europe/London' in str(context.exception))

    def test_given_correct_timezone_when_verify_returns_true(self):
        params = [
            "UTC",
            "Europe/London",
            "US/Hawaii",
        ]
        for i in params:
            with self.subTest(i=i):
                self.assertTrue(Timeseries.verify_timezone(i))



import os
import unittest

from generalindex.utils import PathFormatter
from generalindex.exceptions import MissingArgumentException

os.environ["NG_API_AUTHTOKEN"] = "test"


class UtilsTestCase(unittest.TestCase):

    def test_no_params_when_get_symbols_then_raises_exc(self):
        formatter = PathFormatter()
        with self.assertRaises(MissingArgumentException) as context:
            formatter.get_symbol_queried_path()

        self.assertTrue(
            'Please specify either group_name or search_for parameter of get_symbols() method'
            in str(context.exception))

    def test_given_group_name_when_get_symbols_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbol_queried_path(group_name="MyGroup")
        self.assertTrue(response is not None)
        self.assertEqual(response, '/ts/symbol/search?size=1000&from=0&query=groupName.keyword%3DMyGroup')

    def test_given_search_for_when_get_symbols_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbol_queried_path(search_for="symbols.code=GX0000001")
        self.assertTrue(response is not None)
        self.assertEqual(response, '/ts/symbol/search?size=1000&from=0&query=symbols.code%3DGX0000001')

    def test_given_both_when_get_symbols_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbol_queried_path(group_name="MyGroup", search_for="symbols.code=GX0000001")
        self.assertTrue(response is not None)
        self.assertEqual(response,
                         '/ts/symbol/search?size=1000&from=0&query=symbols.code%3DGX0000001*%26groupName.keyword%3DMyGroup')

    def test_no_params_when_get_symbols_stream_then_raises_exc(self):
        formatter = PathFormatter()
        with self.assertRaises(MissingArgumentException) as context:
            formatter.get_symbols_stream_queried_path()

        self.assertTrue(
            'Please specify either group_name or search_for parameter of get_symbols() method'
            in str(context.exception))

    def test_given_group_name_when_get_symbols_stream_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbols_stream_queried_path(group_name="MyGroup")
        self.assertTrue(response is not None)
        self.assertEqual(response, '/ts/symbol/search/stream?query=groupName.keyword%3DMyGroup')

    def test_given_search_for_when_get_symbols_stream_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbols_stream_queried_path(search_for="symbols.code=GX0000001")
        self.assertTrue(response is not None)
        self.assertEqual(response, '/ts/symbol/search/stream?query=symbols.code%3DGX0000001')

    def test_given_both_when_get_symbols_stream_then_parses_correctly(self):
        formatter = PathFormatter()

        response = formatter.get_symbols_stream_queried_path(group_name="MyGroup", search_for="symbols.code=GX0000001")
        self.assertTrue(response is not None)
        self.assertEqual(response,
                         '/ts/symbol/search/stream?query=symbols.code%3DGX0000001*%26groupName.keyword%3DMyGroup')



import json
import os
import re
from ast import literal_eval

from deprecated import deprecated

from .DatalakeHandler import DatalakeHandler
from .HTTPCaller import HTTPCaller
from .utils import log


@deprecated(version='0.1.20', reason="You should use trip-task-util package instead")
class TaskHandler:
    def __init__(self):
        self.pipeline_id = os.environ.get('PIPELINE_ID')
        self.caller = HTTPCaller()
        self.main_path = os.getcwd()

    # ---------------
    # BASE METHODS
    # ---------------
    def get_storage_value(self, key):
        '''Read a value from the Storage with a key
        :param: key: the key to get the value from storage

        :return: dictionary {'dataStorageId': ..., ...,  'key': 'A', 'value': '1'}
        '''

        path = f'/mgmt/store/{self.pipeline_id}/{key}'

        response = self.caller.get(path)
        return response.json()

    def set_storage_value(self, key, value):
        ''' Write a value in the storage with a corresponding key
        :param: key: the key to get the value from storage
        :param: value to set
        '''

        path = '/mgmt/store/{}/{}'.format(self.pipeline_id, key)
        headers = {'Content-Type': 'text/plain'}

        return self.caller.post(path, headers=headers, data=value)

    # -------------------------------------
    # Task Input and Output related methods
    # -------------------------------------
    def read_task_parameter_value(self, arg_name):
        '''
        Read the input value of a TASK given its name,
        and return the corresponding file JSON that can contain its ID, name or groupName.

        If the file was passed by a previous Task (dynamically) the value should contain its ID (at least), name and groupName
        If the file was selected using the File Picker with the latest
        version of the file, then no ID is passed

        :param: arg_name: name of the TASK parameter to be read from env variables
        :return: returns a dictionary that can contain name, groupName and id of a file ti use as the Task input
        '''

        # Get the Input value from the Env Variables
        arg_key = os.environ.get(arg_name)
        log.debug(f'Raw input for {arg_name}: {arg_key}. Type is: {type(arg_key)}')

        # this is dynamic parameter - get value from storage
        if arg_key.startswith("${"):
            # retrieve the saved value passed dynamically
            value = self.get_storage_value(key=arg_key)['value']

        # this is local value from FilePicker
        else:
            value = arg_key

        log.debug(f'Final Value for parameter {arg_name}: {value}')
        return value

    def write_task_parameter_value(self, output_name, value):
        '''
        Pass the given value in the Storage for the next TASK input

        :param: output_name: name of the Output parameter of the TASK to write for the next task
        :param: value: value to set with the corresponding key
        '''

        # Get the reference for the dynamic output
        output_key = os.environ.get(output_name)

        if output_key is None:
            raise ValueError(
                'No key matching {} - Either wrong Dynamic output name or not correctly passed on the UI'.format(
                    output_name))

        # Write the key - file id in the storage
        self.set_storage_value(output_key, value)
        log.debug(f'File data {value} written in Storage for {output_name} argument with key {output_key}')

        return {output_name: (output_key, value)}

    # -------------------------------------------------------------
    # Combined Methods
    # 1. download file given input parameter name
    # 2. upload file and put file dictionary to the storage
    # -------------------------------------------------------------
    def download_from_input_parameter(self, arg_name, dest_file_name=None, save=False):
        '''
        Download the file using Parameter name and save it as dest_file_name

        :param: output_name: name of the Output parameter of the TASK to write for the next task
        :param: dest_file_name: name of the saved file including the file extension. If None it saves with original name
        :param: save: if True it saves locally the file from the input, otherwise it returns the BytesIO object (streamed from the datalake)

        :return: None if the file is saved, BytesIO object of the file if save i False
        '''

        # Read the passed Input value from the Pipeline Storage
        value = self.read_task_parameter_value(arg_name)
        log.debug(f'Arg_name: {arg_name}, Value: {value}')

        if value in [None, '']:
            raise Exception(f'Nothing found in the parameter {arg_name}: {value}')

        else:
            # Try to read the string value as a dictionary (containing the name, id, groupname keys) or keep as string
            try:
                file_dict = literal_eval(value)
                log.debug('{} reads as dict'.format(value))
            except:
                file_dict = value
                log.debug('{} reads as string'.format(value))

            if isinstance(file_dict, str):
                # Check if a simple ID was passed as value (for BACKWARD COMPATIBILITY)
                regex_id = '[a-z0-9]{8}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{4}-[a-z0-9]{12}'

                if re.search(regex_id, file_dict):
                    # Re-format as dictionary with id key
                    file_dict = {"id": file_dict}

                else:
                    raise Exception('Wrongly formatted value passed as Input: {}'.format(value))

        log.debug(f"Input Value Retrieved - File Dict: {file_dict}")

        # Download the corresponding file
        dh = DatalakeHandler()

        if 'id' in file_dict.keys():
            log.debug('Downloading file by id')

            # Download by ID
            file_id = file_dict['id']

            # Return None if file saved on disk else BytesIO object
            return dh.download_by_id(file_id=file_id, dest_file_name=dest_file_name, save=save)


        elif 'name' in file_dict.keys():
            log.debug('Downloading file by name')

            # Download by name & group
            file_group = file_dict['groupName'] if 'groupName' in file_dict.keys() else None
            file_name = file_dict['name']

            # Return None if file saved on disk else BytesIO object
            return dh.download_by_name(file_name=file_name,
                                       group_name=file_group,
                                       dest_file_name=dest_file_name,
                                       save=save)

        else:
            raise Exception(f'File cannot be downloaded by given parameter name: {arg_name}')

    def upload_to_output_parameter(self, output_name, file, group_name, file_upload_name=None, file_type='SOURCE',
                                   partial_update=False, avoid_duplicates=False):
        '''
        Upload the file to the datalake as the Output of the TASK
        and pass its ID in the Storage for the next TASK input

        :param: output_name: name of the Output parameter of the TASK to write for the next task
        :param: file: Either the object to stream to the datalake, either the path of the file to upload to the datalake
        :param: group_name: group where to upload the output file
        :param: file_upload_name: name of the file to upload on the datalake
                    - MANDATORY if the file is a python object and not a path to a file saved on the disk
        :param: file_type: file type on the datalake among SOURCE, NCSV ...
         :param: partial_update: if true additional columns will be added to the datalake (only NCSV)
        :param: avoid_duplicates: if true duplicated values will not be saved in datalake (only NCSV)
        '''

        # Upload the file to datalake
        uploader = DatalakeHandler()

        file_id, file_name = uploader.upload_file(file=file,
                                                  group_name=group_name,
                                                  file_upload_name=file_upload_name,
                                                  file_type=file_type,
                                                  partial_update=partial_update,
                                                  avoid_duplicates=avoid_duplicates)

        log.debug('File {} uploaded to {} with ID {}'.format(file, group_name, file_id))

        # Write the json containing the id, name, groupName in the Pipeline Storage
        try:
            value = json.dumps({'id': file_id, 'name': file_name, 'groupName': group_name})
            return self.write_task_parameter_value(output_name=output_name, value=value)

        except ValueError:
            log.warning(f'Output {output_name} not connected to another task -no value written in the Pipeline Storage')
            return



import inspect
import json
from io import BytesIO
from typing import Literal

from .HTTPCaller import HTTPCaller
from .exceptions import ApiResponseException
from .utils import PathFormatter

MetadataVals = Literal["true", "false", "only"]
OrderVals = Literal["asc", "desc"]
OutputTypes = Literal["csv", "json"]


class GxApi:
    @staticmethod
    def get_symbols(search: str = None, group_name: str = 'Prod_Indexes', size=1000, start_from=0):
        '''
        Returns a list of the symbols available on a given group or symbols within search
        '''
        query_url = PathFormatter().get_symbol_queried_path(start=start_from, _size=size, group_name=group_name,
                                                            search_for=search)
        return json.load(_ApiRequest(path=query_url).json())

    @staticmethod
    def stream_symbols(search: str = None, group_name: str = 'Prod_Indexes'):
        '''
        Returns a list of the symbols available on a given group or symbols within search
        '''
        query_url = PathFormatter().get_symbols_stream_queried_path(group_name=group_name, search_for=search)
        return json.load(_ApiRequest(path=query_url).json())

    def index(self,
              code: list = None,
              period: list = None,
              period_type: list = None,
              time_ref: list = None,
              group: str = None,
              module: list = None,
              start_from: str = None,
              to: str = None,
              delta: bool = None,
              metadata: MetadataVals = None,
              metadata_fields: dict = None,
              order: OrderVals = None,
              timezone: str = None,
              fields: dict = None,
              headers: dict = None):
        """
        The method retrieves the data from Gx Api based on provided parameters.

        :param code: Symbol code (default '*' )
        :param period: Symbol period (default '*' ).
        :param period_type: Symbol periodType (default '*' )
        :param time_ref: Symbol timeRef (default '*' )
        :param group: Name of the group (default Prod_Indexes)
        :param module: Name of the IndexModule. Can't use with 'code','period','periodType' and 'timeRef' parameter
        :param start_from: Start date (default beginning of the day) Accepted values:
            today
            *d - specific amount of days
            *m - specific amount of months
            *y - specific amount of years
            all- no specified starting point
            {x}v- only x last values for each index
            ISO date ex. 2022-11-21T00:00:00
            where * is a number
        :param to: End date (default end of the day) Accepted values:
            today
            *d - specific amount of days
            *m - specific amount of months
            *y - specific amount of years
            ISO date ex. 2022-11-21T00:00:00
            where * is a number

        :param delta: Delta will filter timeseries by insertion date instead of the actual timestamp of data (default false)
        :param metadata: Metadata (default false) Accepted values: true, false, only
        :param metadata_fields: Filter by metadata field e.g. {"metadata.tradinghub":"NWE"}
        :param order: Order of results by date (default desc)
        :param timezone: Timezone as string used for the timestamps in the Date column.
        Continent/City, for example Europe/London (default UTC)
        :param fields: Filter by value field e.g. {"FactsheetVersion":"1"}
        :param headers: Additional headers, which will be sent with request for symbols e.g. {"Example-Header":"Example_header_value"}

        :return: _ApiRequest
        """
        sig, method_locals = inspect.signature(self.index), locals()
        query_params_dict = {self._to_camel_case(param.name): self._concatenate(method_locals[param.name]) for param in
                             sig.parameters.values() if
                             (method_locals[param.name] is not None and param.name != 'headers')}
        query_params_dict = self._create_query_params(query_params_dict)
        return _ApiRequest(path='/index', query_params=query_params_dict, headers=headers)

    def catalogue(self, code: list = None, no_module: bool = None, module: str = None,
                  fields: dict = None, limit_fields: list = None):
        """
        The method retrieves the data from Gx Api based on provided parameters.

        :param code: Symbol code (default '*' )
        :param no_module: Include indexes without index module set (default false)
        :param module: Filter by provided IndexModule name
        :param fields: Filter by catalogue field e.g. {"field.status":"Live"}
        :param limit_fields: Limit request to specified fields e.x. 'Code,Title'

        :return: _ApiRequest
        """
        sig, method_locals = inspect.signature(self.catalogue), locals()
        query_params_dict = {self._to_camel_case(param.name): self._concatenate(method_locals[param.name]) for param in
                             sig.parameters.values() if
                             (method_locals[param.name] is not None) & (param.name != 'limit_fields')}
        query_params_dict = self._create_query_params(query_params_dict)
        limit_header = {}
        if limit_fields is not None:
            limit_header = {"Fields": self._concatenate(limit_fields)}
        return _ApiRequest(path='/catalogue', query_params=query_params_dict, headers=limit_header)

    def _create_query_params(self, query_params_dict):
        self._flat_map(query_params_dict, "MetadataFields")
        self._flat_map(query_params_dict, "Fields")

        return query_params_dict

    @staticmethod
    def _concatenate(params):
        if type(params) is not list:
            return params
        else:
            query = params.pop(0)
            for s in params:
                query += ','
                query += s

            return query

    @staticmethod
    def _flat_map(query_params_dict, key):
        _dict = query_params_dict.pop(key, None)
        if _dict:
            query_params_dict.update(_dict)

    @staticmethod
    def _to_camel_case(name):
        if name == 'start_from':
            name = 'from'
        return "".join(x.capitalize() for x in name.lower().split("_"))


class _ApiRequest:
    def __init__(self, path, query_params=None, headers: dict = None):
        self._caller = HTTPCaller()
        self._headers = headers
        self._path = path
        self._query_params = query_params

    def csv(self):
        self._update_headers({"Accept": 'text/plain'})
        response = self._caller.get(self._path, headers=self._headers, query_params=self._query_params)
        return self.verify_response(response)

    def json(self):
        self._update_headers({"Accept": 'application/json'})
        response = self._caller.get(self._path, headers=self._headers, query_params=self._query_params)
        return self.verify_response(response)

    def file(self, file_name, output_type: OutputTypes = "csv"):
        if output_type == "csv":
            self._update_headers({"Accept": 'text/plain'})
        else:
            self._update_headers({"Accept": 'application/json'})

        response = self._caller.get(self._path, headers=self._headers, query_params=self._query_params)
        file_name = file_name if '.' in file_name else file_name + '.' + output_type
        with open(file_name, "wb") as text_file:
            text_file.write(self.verify_response(response).read())

    @staticmethod
    def verify_response(response):
        if response.status_code == 200:
            return BytesIO(response.content)
        else:
            raise ApiResponseException("Response code {}.\n {}".format(response.status_code, response.text))

    def request_params(self):
        return self._query_params

    def request_headers(self):
        return self._headers

    def _update_headers(self, param):
        if self._headers is None:
            self._headers = {}
        self._headers.update(param)



import os
import unittest
from io import BytesIO
from unittest import mock
from unittest.mock import Mock

from generalindex.HTTPCaller import HTTPCaller
from generalindex.exceptions import ApiResponseException

os.environ["NG_API_AUTHTOKEN"] = "test"


class ApiTestCase(unittest.TestCase):

    @mock.patch('requests.post')
    def test_given_response_200_when_post_then_returns_response(self, mock_requests):
        caller = HTTPCaller()
        mock_response = Mock()
        mock_requests.return_value = mock_response
        mock_response.status_code = 200

        response = caller.post("https://test.com")

        self.assertTrue(response is not None)

    @mock.patch('requests.post')
    def test_given_response_401_when_post_then_returns_response(self, mock_requests):
        caller = HTTPCaller()
        mock_response = Mock()
        mock_requests.return_value = mock_response
        mock_response.status_code = 401
        with self.assertRaises(ApiResponseException) as context:
            caller.post("https://test.com")

        self.assertTrue('Posting with message None ended with error ' in str(context.exception))

    @mock.patch('requests.post')
    def test_given_headers_when_post_then_updates_accordingly(self, mock_requests):
        caller = HTTPCaller()
        headers = {'Content-Type': 'text/plain'}
        headers_updated = {'Authorization': 'Basic test', 'Content-Type': 'text/plain',
                           'User-Agent': 'GX_PythonSDK_V0.1.13'}
        mock_response = Mock()
        mock_requests.return_value = mock_response
        mock_response.status_code = 200

        response = caller.post("https://test.com", headers=headers)

        self.assertTrue(response is not None)
        calls = mock_requests.mock_calls
        self.assertDictEqual(
            calls[0].kwargs['headers'], headers_updated)

    @mock.patch('requests.Session')
    def test_given_headers_when_post_then_updates_accordingly(self, mock_session):
        caller = HTTPCaller()
        headers = {'Content-Type': 'text/plain'}
        headers_updated = {'Authorization': 'Basic test', 'Content-Type': 'text/plain',
                           'User-Agent': 'GX_PythonSDK_V0.1.13'}
        response_mock = Mock()
        mock_session.return_value.get.return_value = response_mock
        response_mock.url = 'test'
        response_mock.status_code = 200

        response = caller.get("https://test.com", headers=headers)

        self.assertTrue(response is not None)
        self.assertEqual(mock_session.return_value.mount.call_count, 2)
        calls = mock_session.return_value.get.mock_calls
        self.assertDictEqual(
            calls[0].kwargs['headers'], headers_updated)

    @mock.patch('requests.Session')
    def test_given_401_when_post_then_updates_accordingly(self, mock_session):
        caller = HTTPCaller()
        headers = {'Content-Type': 'text/plain'}
        headers_updated = {'Authorization': 'Basic test', 'Content-Type': 'text/plain',
                           'User-Agent': 'GX_PythonSDK_V0.1.13'}
        response_mock = Mock()
        mock_session.return_value.get.return_value = response_mock
        response_mock.url = 'test'
        response_mock.status_code = 401

        with self.assertRaises(ApiResponseException) as context:
            caller.get("https://test.com", headers=headers)
            self.assertEqual(mock_session.return_value.mount.call_count, 2)
            calls = mock_session.return_value.get.mock_calls
            self.assertDictEqual(
                calls[0].kwargs['headers'], headers_updated)

        self.assertTrue('Data reception from https://test.com ended with error' in str(context.exception))

    @mock.patch('requests.Session')
    @mock.patch('requests.put')
    @mock.patch('requests.post')
    def test_given_file_when_upload_then_returns_id(self, mock_post, mock_put, mock_session):
        caller = HTTPCaller()
        file_name = "test.json"
        p_u = [
            {"name": "PARTIAL_UPDATE", "value": "true"},
            {"name": "AVOID_DUPLICATES", "value": "true"}]
        payload = {"groupName": "group_name",
                   "fileName": file_name,
                   "fileType": "SOURCE",
                   "fields": p_u}

        headers = {'Content-Type': 'text/plain'}
        headers_updated = {'Authorization': 'Basic test', 'Content-Type': 'text/plain',
                           'User-Agent': 'GX_PythonSDK_V0.1.13'}
        file = BytesIO(bytes("{tes:v}", 'utf-8'))
        post_response_mock = Mock()
        put_response_mock = Mock()
        get_response_mock = Mock()
        mock_post.return_value = post_response_mock
        post_response_mock.status_code = 200
        file_id = 12
        post_response_mock.json.return_value = {
            'location': 1,
            'fileId': file_id
        }
        mock_put.return_value = put_response_mock
        put_response_mock.status_code = 200

        mock_session.return_value.get.return_value = get_response_mock
        get_response_mock.status_code = 200
        get_response_mock.json.return_value = {
            'items': [
                {
                    "componentStatuses": [
                        {
                            "status": "INFO",
                            "payload": {
                                "fileId": file_id
                            }
                        }]
                }
            ],

        }
        response = caller.file_uploader(payload=payload, file=file, headers=headers)

        self.assertEqual(response, file_id)
        calls = mock_post.mock_calls
        self.assertDictEqual(
            calls[0].kwargs['headers'], headers_updated)

    @mock.patch('requests.Session')
    def test_given_file_not_uploaded_when_upload_then_raises_exc(self, mock_session):
        caller = HTTPCaller()
        get_response_mock = Mock()
        mock_session.return_value.get.return_value = get_response_mock
        get_response_mock.status_code = 200
        get_response_mock.json.return_value = {
            'items': [],
        }

        result = caller.check_upload_successful(avoid_duplicates=False, job_id='12', file_id='ff',
                                                uploaded_file_check_retries=1)

        self.assertEqual(result, False)

    @mock.patch('requests.Session')
    def test_given_avoid_duplicates_false_when_check_upload_then_returns_accordingly(self, mock_session):
        caller = HTTPCaller()
        get_response_mock = Mock()
        mock_session.return_value.get.return_value = get_response_mock
        get_response_mock.status_code = 200
        params = [
            [False, []],
            [False, [
                {
                    "componentStatuses": []
                }
            ]],
            [False, [
                {
                    "componentStatuses": [
                        {
                            "status": "WARN",
                            "payload": {
                                "fileId": "ff"
                            }
                        }
                    ]
                }
            ]],
            [True, [
                {
                    "componentStatuses": [
                        {
                            "status": "INFO",
                            "payload": {
                                "fileId": "ff"
                            }
                        }
                    ]
                }
            ]]
        ]
        for i in params:
            with self.subTest(i=i):
                get_response_mock.json.return_value = {
                    'items': i[1],
                }

                result = caller.check_upload_successful(avoid_duplicates=False, job_id='12', file_id='ff',
                                                        uploaded_file_check_retries=1)

                self.assertEqual(i[0], result)

    @mock.patch('requests.Session')
    def test_given_avoid_duplicates_when_check_upload_then_returns_accordingly(self, mock_session):
        caller = HTTPCaller()
        get_response_mock = Mock()
        mock_session.return_value.get.return_value = get_response_mock
        get_response_mock.status_code = 200
        params = [

            [True, [
                {"componentStatuses": [
                    {
                        "status": "WARN",
                        "payload": {
                            "fileId": "ff"
                        }
                    }
                ]
                }
            ]],
            [True, [
                {"componentStatuses": [
                    {
                        "status": "INFO",
                        "payload": {
                            "fileId": "ff"
                        }
                    }
                ]
                }
            ]]
        ]
        for i in params:
            with self.subTest(i=i):
                get_response_mock.json.return_value = {
                    'items': i[1],
                }

                result = caller.check_upload_successful(avoid_duplicates=True, job_id='12', file_id='ff',
                                                        uploaded_file_check_retries=1)

                self.assertEqual(i[0], result)

    @mock.patch('requests.post')
    def test_given_wrong_file_when_upload_then_raise_exc(self, mock_post):
        caller = HTTPCaller()
        post_response_mock = Mock()

        mock_post.return_value = post_response_mock
        post_response_mock.status_code = 200
        file_id = 12
        post_response_mock.json.return_value = {
            'location': 1,
            'fileId': file_id
        }
        with self.assertRaises(TypeError) as context:
            caller.file_uploader(payload=None, file=123)

        self.assertTrue(
            'Passed file argument must be either a file path to a file saved on the disk, either a io.BytesIO object to stream from memory' in str(
                context.exception))

    def test_given_avoid_duplicates_return_true(self):
        caller = HTTPCaller()

        p_u = [
            {"name": "PARTIAL_UPDATE", "value": "true"},
            {"name": "AVOID_DUPLICATES", "value": "true"}]

        payload = {"fields": p_u}
        self.assertTrue(caller.is_avoid_duplicates(payload))

    def test_given_not_avoid_duplicates_return_true(self):
        caller = HTTPCaller()
        p_u = [{"name": "AVOID_DUPLICATES", "value": "false"},
               {"name": "PARTIAL_UPDATE", "value": "true"}]
        payload = {"fields": p_u}
        self.assertEqual(caller.is_avoid_duplicates(payload), False)



import logging
import sys
from urllib import parse

from .constants import LOGGER_NAME
from .exceptions import MissingArgumentException

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
log = logging.Logger(LOGGER_NAME)

logger_handler = logging.StreamHandler()
logger_handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
log.addHandler(logger_handler)


def is_blank(param):
    return param in ["", None]


class PathFormatter:
    def get_symbol_queried_path(self, _size=1000, start=0, group_name=None, search_for=None):
        formatted_query = self._get_symbol_queries(group_name, search_for)
        return '/ts/symbol/search?size={}&from={}&query={}'.format(_size, start,
                                                                   formatted_query)

    def get_symbols_stream_queried_path(self, group_name=None, search_for=None):
        formatted_query = self._get_symbol_queries(group_name, search_for)
        return '/ts/symbol/search/stream?query={}'.format(formatted_query)

    def _get_symbol_queries(self, group_name, search_for):

        if (is_blank(group_name)) and (is_blank(search_for)):
            raise MissingArgumentException(
                'Please specify either group_name or search_for parameter of get_symbols() method')

        formatted_group = self._parse_symbol_query(group_name)
        formatted_search = self._parse_symbol_query(search_for)

        if not is_blank(formatted_search) and not is_blank(formatted_group):
            return '{}*%26groupName.keyword%3D{}'.format(
                formatted_search,
                formatted_group)
        elif not is_blank(formatted_search):
            return formatted_search
        else:
            return 'groupName.keyword%3D{}'.format(
                formatted_group)

    @staticmethod
    def _parse_symbol_query(symbol_query):
        if not is_blank(symbol_query):
            return parse.quote(symbol_query)
        else:
            return None



import unittest

from generalindex.Authenticator import Authenticator


class ApiTestCase(unittest.TestCase):

    def test_given_basic_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        auth.token = "token"

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "Basic token")

    def test_given_prefixed_basic_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        auth.token = "basic token"

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "basic token")

    def test_given_prefixed_bearer_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        auth.token = "bearer token"

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "bearer token")

    def test_given_bearer_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        token = "dada4rwfsrfe5fscscja9238ds3ea9isfwkh1i7y387y4o87egfho9y03y74fy7yao87doq8373tgeo87gwo8f7tz6xfai6eti6qtdi6atxo6ateq6te"
        auth.token = token

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "Bearer " + token)

    def test_given_prefixed_apikey_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        auth.token = "apikey token"

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "apikey token")

    def test_given_apikey_token_when_get_token_then_returns_proper(self):
        auth = Authenticator()
        auth.token = "NG_123_dcsdc"

        self.assertTrue(auth.get_token() is not None)
        self.assertTrue(auth.get_token().get("Authorization") == "ApiKey NG_123_dcsdc")



# Import the methods
from .DatalakeHandler import DatalakeHandler
from .GxApi import GxApi
from .StatusHandler import StatusHandler
from .TaskHandler import TaskHandler
from .Timeseries import Timeseries
# Set logger with proper format for backend parsing to status page



import os
import unittest
from unittest.mock import Mock, patch

from generalindex.GxApi import GxApi, _ApiRequest
from generalindex.exceptions import ApiResponseException

os.environ["NG_API_AUTHTOKEN"] = "test"


class ApiTestCase(unittest.TestCase):
    def test_given_dict_params_when_index_called_with_parsed_params(self):
        api = GxApi()
        param_dict = {'PeriodType': 'Month', 'TimeRef': '1200', 'Code': 'GX0123', 'From': '1v',
                      'metadata.test': 'value', 'metadata.test2': 'value2', 'Period': '1', 'testField': 'test'}

        response = api.index(["GX0123"], ["1"], ["Month"], time_ref=["1200"], start_from="1v",
                             metadata_fields={"metadata.test": "value", "metadata.test2": "value2"},
                             fields={'testField': "test"})
        self.assertTrue(response is not None)
        self.assertEqual(response.request_params(), param_dict)

    def test_given_dict_params_when_catalogue_called_with_parsed_params(self):
        api = GxApi()
        param_dict = {'Code': 'GX0123,GX01233', 'field.testField': 'test', 'NoModule': True, 'Module': 'TestModule'}

        catalogue = api.catalogue(["GX0123", "GX01233"], no_module=True, module='TestModule',
                                  fields={'field.testField': "test"}, limit_fields=["Code", "Test"])
        self.assertTrue(catalogue is not None)
        self.assertEqual(catalogue.request_params(), param_dict)
        self.assertEqual(catalogue.request_headers(), {"Fields": "Code,Test"})

    def test_given_query_params_when_csv_then_returns_stream(self):
        param_dict = {'PeriodType': 'Month', 'TimeRef': '1200', 'code': 'GX0123', 'from': '1v',
                      'metadata.test': 'value', 'period': '1', 'testField': 'test'}
        response_text = """Code,PeriodType,TimeRef,IndexModule,Alias,Bid,Commodity,Currency,DeliveryBasis,EndDate,
        FactsheetVersion,Frequency,Group,High,HolidayCalendar,Index,InterAffiliateDataAccepted,LastTradeDateCalendar,
        Low,MapLocation,Mid,Offer,ParentCodes,PeriodMax,PeriodMin,PermissionLatestStatus,Precision,PricingBasis,
        PriorityToTransactions,QaSeries,Quantity,SoleSourcedDataAccepted,Source,StartDate,Status,TimeRefDetails,
        TimeZone,Title,TradingHub,Units,Volume GX0006208,Prompt,1630,AmericasMarine,,,Marine Fuel,USD,Delivered,,2,
        Daily,Prod_Indexes,,Holidays_ICE_Brent,Y,N,,,20.6542084 -105.2458316,,,GX0002016,1,1,DEVELOPMENT,3,Flat,Y,,,
        N,GX,,New,Houston Close,America/Chicago,Intermediate Fuel Oil 180 CST Bunker Puerto Vallarta (MXPVR) Mexico,
        North America,MT,"""
        index = _ApiRequest('/index', param_dict)

        mock_caller = Mock()
        mock_caller.get.return_value.status_code = 200
        mock_caller.get.return_value.content = str.encode(response_text)

        with patch.object(index, "_caller", new=mock_caller):
            result = index.csv()
            calls = mock_caller.mock_calls
            self.assertTrue(result is not None)
            self.assertEqual(str(result.read(), encoding='utf-8'), response_text)
            self.assertDictEqual(
                calls[0].kwargs['query_params'], param_dict)
            self.assertDictEqual(
                calls[0].kwargs['headers'], {"Accept": "text/plain"})

    def test_given_status_401_when_csv_then_raises_exc(self):
        param_dict = {}
        response_text = """{
    "timestamp": "2023-12-19T10:03:43.246402281",
    "message": "User not authorized."}"""

        mock_caller = Mock()
        mock_caller.get.return_value.status_code = 503
        mock_caller.get.return_value.text = response_text
        index = _ApiRequest('/index', param_dict)
        with patch.object(index, "_caller", new=mock_caller):
            with self.assertRaises(ApiResponseException) as context:
                index.csv()

            self.assertTrue(
                'Response code 503.\n {\n    "timestamp": "2023-12-19T10:03:43.246402281",\n    "message": "User not '
                'authorized."}' in str(
                    context.exception))

    def test_given_query_params_when_json_then_returns_stream(self):
        param_dict = {'PeriodType': 'Month', 'TimeRef': '1200', 'code': 'GX0123', 'from': '1v',
                      'metadata.test': 'value', 'period': '1', 'testField': 'test'}

        response_text = """[
            {
                "Code": "GX0007205",
                "PeriodType": "Prompt",
                "TimeRef": "1630",
                "IndexModule": "EuropeanMarine",
                "Commodity": "Marine Fuel",
                "Currency": "USD",
                "DeliveryBasis": "Delivered",
                "FactsheetVersion": "2",
                "Frequency": "Daily",
                "Group": "Prod_Indexes",
                "HolidayCalendar": "Holidays_ICE_Brent",
                "Index": "Y",
                "InterAffiliateDataAccepted": "N",
                "MapLocation": "5.860754324 -10.05227963",
                "ParentCodes": "GX0000544",
                "PeriodMax": "1",
                "PeriodMin": "1",
                "PermissionLatestStatus": "DEVELOPMENT",
                "Precision": "3",
                "PricingBasis": "Flat",
                "PriorityToTransactions": "Y",
                "SoleSourcedDataAccepted": "N",
                "Source": "GX",
                "Status": "New",
                "TimeRefDetails": "London Close",
                "TimeZone": "Europe/London",
                "Title": "Marine Gasoil Bunker Buchanan (LRUCN) Liberia",
                "TradingHub": "West Africa",
                "Units": "MT"
            }
            ]"""
        index = _ApiRequest('/index', param_dict)
        mock_caller = Mock()
        mock_caller.get.return_value.status_code = 200
        mock_caller.get.return_value.content = str.encode(response_text)

        with patch.object(index, "_caller", new=mock_caller):
            result = index.json()
            calls = mock_caller.mock_calls
            self.assertTrue(result is not None)
            self.assertEqual(str(result.read(), encoding='utf-8'), response_text)
            self.assertDictEqual(
                calls[0].kwargs['query_params'], param_dict)
            self.assertDictEqual(
                calls[0].kwargs['headers'], {"Accept": "application/json"})

    def test_given_json_when_file_then_returns_stream(self):
        param_dict = {'PeriodType': 'Month', 'TimeRef': '1200', 'code': 'GX0123', 'from': '1v',
                      'metadata.test': 'value', 'period': '1', 'testField': 'test'}

        response_text = """[
            {
                "Code": "GX0007205",
               }
            ]"""
        index = _ApiRequest('/index', param_dict)
        mock_caller = Mock()
        mock_caller.get.return_value.status_code = 200
        mock_caller.get.return_value.content = str.encode(response_text)

        with patch.object(index, "_caller", new=mock_caller):
            index.file("/tmp/file", "json")
            calls = mock_caller.mock_calls

            self.assertDictEqual(
                calls[0].kwargs['query_params'], param_dict)
            self.assertDictEqual(
                calls[0].kwargs['headers'], {"Accept": "application/json"})

    def test_given_csv_params_when_file_then_returns_stream(self):
        param_dict = {'PeriodType': 'Month', 'TimeRef': '1200', 'code': 'GX0123', 'from': '1v',
                      'metadata.test': 'value', 'period': '1', 'testField': 'test'}
        response_text = "Code"
        index = _ApiRequest('/index', param_dict)
        mock_caller = Mock()
        mock_caller.get.return_value.status_code = 200
        mock_caller.get.return_value.content = str.encode(response_text)

        with patch.object(index, "_caller", new=mock_caller):
            index.file("/tmp/file", )
            calls = mock_caller.mock_calls
            self.assertDictEqual(
                calls[0].kwargs['query_params'], param_dict)
            self.assertDictEqual(
                calls[0].kwargs['headers'], {"Accept": "text/plain"})



import os

import setuptools


class CleanCommand(setuptools.Command):
    """Custom clean command to tidy up the project root."""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open('requirements.txt') as fin:
    requirements = fin.read()

setuptools.setup(
    name="generalindex",
    version="0.2.6",
    author="General Index",
    author_email="info@general-index.com",
    description="Python SDK for the General Index platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.general-index.com/",
    packages=setuptools.find_packages(exclude=["test"]),
    classifiers=["Programming Language :: Python :: 3",
                 "License :: OSI Approved :: MIT License",
                 "Operating System :: OS Independent"
                 ],
    install_requires=[
        requirements
    ],
    python_requires='>=3.9',
    cmdclass={
        'clean': CleanCommand,
    }
)
